
' DS2406 is a VB.Net class that demonstrates how to work with a
' Dallas Semiconductor DS2406 1-Wire addressable switch using
' an EDS 1-Wire Bus Master.
' 
' HARDWARE DESCRIPTION
' 
' The DS2406 Dual Addressable Switch Plus Memory offers a
' simple way to remotely control a pair of open drain
' transistors and to monitor the logic level at each
' transistors output via the 1-Wire bus for closed loop
' control. Each DS2406 has its own 64-bit ROM registration
' number that is factory lasered into the chip to provide a
' guaranteed unique identity for absolute traceability. The
' devices 1024 bits of EPROM can be used as electronic label
' to store information such as switch function, physical
' location, and installation date. Communication with the
' DS2406 follows the standard Dallas Semiconductor 1-Wire
' protocol and can be accomplished with minimal hardware such
' as a single port pin of a microcontroller. Multiple DS2406
' devices can reside on a common 1-Wire network and be operated
' independently of each other. Individual devices will respond
' to a Conditional Search command if they qualify for certain
' user-specified conditions, which include the state of the
' \output transistor, the static logic level or a voltage
' transition at the transistors output.                       
	Public Class DS2406
		Inherits Dallas1WireDevice

		Public Const SOURCE_ACTIVITY_LATCH As Byte = &H2
		Public Const SOURCE_FLIP_FLOP As Byte = &H4
		Public Const SOURCE_PIO As Byte = &H6
		Public Const POLARITY_ZERO As Byte = &H0
		Public Const POLARITY_ONE As Byte = &H1
		Public Const CHANNEL_NONE As Byte = &H0
		Public Const DONT_CHANGE As Byte = &HFF
		Public Const CHANNEL_A_ONLY As Byte = &H4
		Public Const CHANNEL_B_ONLY As Byte = &H8
		Public Const CHANNEL_BOTH As Byte = &HC
		Public Const CRC_DISABLE As Byte = &H0
		Public Const CRC_EVERY_BYTE As Byte = &H1
		Public Const CRC_EVERY_8_BYTES As Byte = &H2
		Public Const CRC_EVERY_32_BYTES As Byte = &H3

		Private Const WRITE_STATUS_COMMAND As Byte = &H55
		Private Const READ_STATUS_COMMAND As Byte = &HAA
		Private Const CHANNEL_ACCESS_COMMAND As Byte = &HF5

		Private ODD_PARITY() As Integer = {0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0}


		Public Sub New(ByVal ROMId As String, ByVal busMaster As OWInterface.BusMasterInterface)
			MyBase.New(ROMId, busMaster)
		End Sub

		' \** NOT IMPLEMENTED**
		' 
		' The Read Memory command is used to read data from the
		' 1024-bit EPROM data memory field. The bus master follows the
		' command byte with a two-byte address (TA1=(T7:T0),
		' TA2=(T15:T8)) that indicates a starting byte location within
		' the data field. Since the data memory contains 128 bytes,
		' T15:T8 and T7 should all be zero. With every subsequent read
		' data time slot the bus master receives data from the DS2406
		' starting at the initial address and continuing until the end
		' \of the 1024-bits data field is reached or until a Reset
		' Pulse is issued.                                            
		Public Function ReadMemory() As String
			'TODO - Implement this function

		End Function
		'ReadMemoryPages is used to read one or more entire memory pages
		'from the DS2046.
		'The return value is an ArrayList of Strings, with each string representing
		'one memory page.
		Public Function ReadMemoryPages(ByVal StartPage As Integer, Optional ByVal numPages As Integer = 1) As ArrayList
			Dim owResponse As OW_Response
			If (StartPage > 3) Or (StartPage + numPages) > 4 Then
				'TODO - Throw an exception due to invalid page numbers requested.
			End If
			With Me.BusMaster
				.OW_GetLock()
				owResponse = .OW_ReadPages(StartPage, numPages, Me.ROMId)
				.OW_ResetBus()
				.OW_ReleaseLock()
			End With
			Return owResponse.Data
		End Function
		' \** NOT IMPLEMENTED**
		' 
		' The Extended Read Memory command supports page redirection
		' when reading data from the 1024-bit EPROM data field. One
		' major difference between the Extended Read Memory and the
		' basic Read Memory command is that the bus master receives the
		' Redirection Byte (see description of Status Memory) first
		' before investing time in reading data from the addressed
		' memory location. This allows the bus master to quickly decide
		' whether to continue and access the data at the selected
		' starting page or to terminate and restart the reading process
		' at the redirected page address.
		' 
		' In addition to page redirection, the Extended Read Memory
		' command also supports bit-oriented applications where the
		' user cannot store a 16-bit CRC with the data itself. With
		' bit-oriented applications the EPROM information may change
		' \over time within a page boundary making it impossible to
		' include an accompanying CRC that will always be valid.
		' Therefore, the Extended Read Memory command concludes each
		' page with the DS2406 generating and supplying a 16-bit CRC
		' that is based on and therefore always consistent with the
		' current data stored in each page of the 1024-bit EPROM data
		' field.                                                       
		Public Function ExtendedReadMemory() As String
			'TODO - Implement this function
		End Function
		' \** NOT IMPLEMENTED**
		' 
		' The Write Memory command is used to program the 1024-bit
		' EPROM data memory. The details of the functional flow chart
		' are described in the section Writing EPROM Memory of the
		' DS2406 DataSheet. The data memory address range is 0000h to
		' 007Fh. If the bus master sends a starting address higher than
		' this, the nine most significant address bits are set to zeros
		' by the internal circuitry of the chip. This will result in a
		' mismatch between the CRC calculated by the DS2406 and the CRC
		' calculated by the bus master, indicating an error condition. 
		Public Function WriteMemory() As String
			'TODO - Implement this function
		End Function
		'  
		' The Write Status command is used to program the Status
		' Memory, which includes the specification of the Conditional
		' Search Settings. The details of the functional flow chart are
		' described in the section Writing EPROM Memory of the DS2406
		' DataSheet.  
		'Returns:  The new status byte as reported by the DS2406
		Public Function WriteStatus(ByVal statusAddress As Byte, ByVal dataByte As Byte) As Byte
			Dim owResponse As OW_Response
			owResponse = Me.BusMaster.OW_WriteBlock(Hex2(WRITE_STATUS_COMMAND) & Hex2(statusAddress) & "00" & Hex2(dataByte) & "FFFF", Me.ROMId)
			Return CByte("&h" & Right(owResponse.Data(0), 2))
		End Function
		' 
		' The Read Status command is used to read data from the Status
		' Memory field. The functional flow chart of this command is
		' identical to the Read Memory command. Since the Status Memory
		' is only 8 bytes, the DS2406 will send the 16-bit CRC after
		' the last byte of status information has been transmitted.    
		'Returns- The status byte read from the DS2406 memoy location
		'specified by statusAddress.
		Public Function ReadStatus(ByVal statusAddress As Byte) As Byte
			Dim owResponse As OW_Response
			Dim statusByte As Byte
			Me.BusMaster.log("Reading DS2406 Status Byte " & CStr(statusAddress) & vbCrLf)
			Me.BusMaster.incrementLogLevel()
			owResponse = Me.BusMaster.OW_WriteBlock(Hex2(READ_STATUS_COMMAND) & Hex2(statusAddress) & "00FF", Me.ROMId)
			statusByte = CByte("&h" & Right(owResponse.Data(0), 2))
			Me.BusMaster.decrementLogLevel()
			Me.BusMaster.log("Status Byte Returned: " & "0x" & Hex2(statusByte) & vbCrLf)
			Return statusByte
		End Function

		' The Channel Access command is used to access the PIO channels
		' to sense the logical status of the output node and the output
		' transistor and to change the status of the output transistor.
		' The bus master will follow the command byte with two Channel
		' Control Bytes and will receive back the Channel Info byte.
		' The Channel Control bytes allow the master to select a
		' PIO-channel to communicate with, to specify communication
		' \parameters, and to reset the activity latches.              

		'Note that this function assumes that any neccesary locks have been acquired,
		'and that the device has been addressed already.

		Public Function ChannelAccess(ByVal bytesToWrite As Byte(), ByVal toggleRW As Boolean, ByVal readInitially As Boolean, ByVal CRCMode As Integer, ByVal ChannelMode As Integer, ByVal ClearActivity As Boolean, ByVal Interleave As Boolean) As String
			Dim owResponse As OW_Response

			CRCMode = CRCMode And &H3			 ' MASK THIS TO ACCEPTABLE VALUE
			ChannelMode = ChannelMode And &HC		 'MASK THIS TO ACCEPTABLE VALUE
			If ChannelMode = 0 Then
				'ChannelMode cannot e 0
				ChannelMode = &H4
			End If
			If (Interleave And (ChannelMode <> CHANNEL_BOTH)) Then
				'Cannot interleave with only 1 channel
				Interleave = False
			End If

			Dim crc As Integer
			Dim i As Integer
			'Now figure out how many bytes the output buffer needs to be
			Dim inLength As Integer = bytesToWrite.Length()
			If toggleRW Then
				inLength = inLength * 2
			End If

			Select Case CRCMode
				Case Is = CRC_EVERY_BYTE
					'Allow for 2 CRC bytes for every byte of the length
					inLength = inLength * 3				'length + 2*length
				Case CRC_EVERY_8_BYTES
					'Allow for 2 CRC bytes for every 8 bytes of length
					inLength = inLength + ((inLength >> 3) << 1)				'(length DIV 8) * 2
				Case CRC_EVERY_32_BYTES
					'Allow for 2 CRC bytes for every 32 bytes of length
					inLength = inLength + ((inLength >> 5) << 1)				'(length DIV 32) * 2
			End Select

			Dim outputbuffer(inLength + 3 + 1 - 1) As Byte		 ' 3 control bytes + 1 information byte
			Dim inbuffer(inLength + 3 + 1 - 1) As Byte
			outputbuffer(0) = CHANNEL_ACCESS_COMMAND
			crc = CRC16((CHANNEL_ACCESS_COMMAND And &HFF))
			'send the control bytes
			outputbuffer(1) = (CRCMode Or ChannelMode Or (IIf(ClearActivity, &H80, 0)) Or (IIf(Interleave, &H10, 0)) Or (IIf(toggleRW, &H20, 0)) Or (IIf(readInitially, &H40, 0)))
			outputbuffer(2) = &HFF
			crc = CRC16(outputbuffer, 1, 2, crc)

			For ij As Integer = 3 To outputbuffer.Length() - 1
				outputbuffer(ij) = &HFF
			Next

			Dim j As Integer = 4		 '   //outputbuffer 0-2 is command bytes, outputbuffer(3) is return info
			Dim options As Integer = outputbuffer(1) And &H63		 'get the bits out we want for toggle, initial, and CRC
			options = ((options >> 3) Or options) And &HF		 'make it a number 0-15
			If ((options < 8) Or (options > &HB)) Then		 'if this is not a read-only (which we need do nothing for)
				For ik As Integer = 0 To bytesToWrite.Length() - 1
					If (options > &HB) Then				'we are reading first
						j = j + fixJ((ik * 2) + 1, options)				 ' leave a space for a read, and the CRC if needed
					End If
					outputbuffer(j) = bytesToWrite(ik)				 'write this data
					If (options < &H4) Then				'if this is only a write
						j = j + fixJ(ik + 1, options)				 'leave a space for CRC if needed, else just increment
					Else				'else we are toggling
						If (options < &H8) Then				 'this is a write-first toggle
							j = j + fixJ((ik * 2) + 1, options)					 'so leave a space for a read
						End If
						j = j + fixJ((ik * 2) + 2, options)				 'leave a space for the CRC
					End If
				Next
			End If

			'now our output buffer should be set correctly
			owResponse = Me.BusMaster.OW_WriteBlock(EDS_OWInterface.ByteArray2Hex(outputbuffer))
			Dim byteCount As Int16
			For t As Integer = 1 To Len(owResponse.Data(0)) Step 2
				inbuffer(byteCount) = CByte("&h" & Mid$(owResponse.Data(0), t, 2))
				byteCount = byteCount + 1
			Next t
			'Me.BusMaster.OW_ReleaseLock()
			'     adapter.dataBlock(outputbuffer, 0, outputbuffer.length);

			''Calculate the CRC16 within the resulting buffer for integrity
			''Start at offset 3 for the information byte
			crc = CRC16(inbuffer(3), crc)
			j = 0		 'j will be how many bytes we are into the buffer - CRC bytes read
			Dim k As Integer = 0		 'index into the return buffer
			Dim fresh As Boolean = False		 ' whether or not we need to reinitialize the CRC calculation
			Dim returnbuffer(bytesToWrite.Length()) As Byte
			For il As Integer = 4 To inbuffer.Length() - 1
				If (CRCMode <> CRC_DISABLE) Then
					If (fresh) Then
						crc = CRC16(inbuffer(il))
						fresh = False
					Else
						crc = CRC16(inbuffer(il), crc)
					End If

					If ((Not (toggleRW) And readInitially) Or (toggleRW And readInitially And ((j And &H1) = &H0)) Or (toggleRW And Not (readInitially) And ((j And &H1) = &H1))) Then
						returnbuffer(k) = inbuffer(il)
						k = k + 1
					End If
					j = j + 1
					If ((fixJ(j, options) > 1) And (CRCMode <> CRC_DISABLE)) Then				 'means that we should look for a CRC then
						crc = CRC16(inbuffer, il + 1, 2, crc)
						il = il + 2

						If (crc <> &HB001) Then
							'1-Wire CRC Exception
							'Stop
							'TODO -Throw an appropriate error
							'Throw New OneWireIOException("Invalid CRC")
						End If
					End If
					fresh = True
				End If
			Next

			'now that we got the right bytes out of the array
			Return owResponse.Data(0)
		End Function
		'Determines whether the DS2406 is being externally powered,
		'or whether it is operating off of power being derived parasitically
		'from the 1-Wire signal pin.
		'NOTE- This seems to always return false, even with the DS2406 is
		'properly connected to 5Vdc Vcc.
		'TODO - Investigate this.
		Public Function ReadPowerSupply() As Boolean
			Dim statusByte As Byte
			Dim powerSupplyBit As Byte
			Dim channelInfoByte As Byte
			Dim retVal As Boolean

			Me.BusMaster.log("DS2406 - Determining Power Supply Status by Examining bit 7 of Status Byte 7." & vbCrLf)
			Me.BusMaster.incrementLogLevel()
			statusByte = Me.ReadStatus(7)
			Me.BusMaster.decrementLogLevel()
			'Me.BusMaster.log("Status byte read from device =  " & CStr(statusByte))

			powerSupplyBit = (statusByte And &H80)
			Me.BusMaster.log("Masking off all bits except bit 7 =  " & CStr(powerSupplyBit) & vbCrLf)

			If powerSupplyBit = 1 Then
				Me.BusMaster.log("DS2406 Vcc pin is connected to suitable power supply." & vbCrLf)
				retVal = True
			Else
				Me.BusMaster.log("DS2406 is operating in parasitic power mode." & vbCrLf)
				retVal = False
			End If
			Return retVal
		End Function
		'Determines the number of PIO channels that are available
		'on this DS2406
		Public Function GetNumberOfChannels() As Integer
			Dim numberOfChannels As Integer
			Dim channelInfoByte As Byte

			channelInfoByte = Me.GetChannelInfoByte()
			'Look at bit 6 to determine the number of Channels
			'If bit 6 = 0, then only channel A is available
			If channelInfoByte And &H40 = 0 Then
				'This is a DS2406 with only 1 channel
				numberOfChannels = 1
			Else
				'This is a DS2406P, with two channels
				numberOfChannels = 2
			End If
			Return numberOfChannels
		End Function
		'Reads the channel info byte from the DS2406 without affecting the PIO lines or latches
		Public Function GetChannelInfoByte() As Byte
			Dim channelInfoByte As Byte
			'Write Channel Control Byte: 01001100
			'This will have no effect on the PIO Lines, but will return the
			'current ChannelInfoByte
			Me.BusMaster.log("GetChannelInfoByte -" & vbCrLf)
			Me.BusMaster.incrementLogLevel()
			Me.BusMaster.log("Writing channel control byte 01001100 to the DS2406." & vbCrLf)
			channelInfoByte = Me.readChannelInfoByte(&H4C)
			Me.BusMaster.decrementLogLevel()
			Me.BusMaster.log("ChannelInfoByte Returned: " & Hex2(channelInfoByte) & vbCrLf)
			Return channelInfoByte
		End Function
		'Resets the activity latches on the DS2406 without otherwise affecting the PIO lines
		'Returns the resulting ChannelInfoByte
		Public Function ResetActivityLatches() As Byte
			Dim channelInfoByte As Byte
			''Write Channel Control Byte: 11001100
			''This will have no effect on the PIO Lines, but will return the
			''current ChannelInfoByte
			'channelInfoByte = Me.readChannelInfoByte(&HCC)

			Dim dummyByte() As Byte = {&HFF}
			Dim owResponse As String
			Me.BusMaster.OW_GetLock()
			Me.BusMaster.OW_AddressDevice(Me.ROMId)
			owResponse = Me.ChannelAccess(dummyByte, False, True, CRC_EVERY_BYTE, CHANNEL_A_ONLY, True, False)
			Me.BusMaster.OW_ReleaseLock()

			channelInfoByte = CByte("&h" & Right(owResponse, 2))

			Return channelInfoByte

		End Function
		'Writes the ChannelControlByte to the DS2406 using the DS2406 ChannelAccess
		'command.
		'Returns the ChannelInfoByte
		Private Function readChannelInfoByte(ByVal ChannelControlByte As Byte) As Byte
			Dim channelInfoByte As Byte
			Dim dummyByte() As Byte = {&HFF}
			Dim owResponse As String
			Me.BusMaster.OW_GetLock()
			Me.BusMaster.OW_AddressDevice(Me.ROMId)
			owResponse = Me.ChannelAccess(dummyByte, False, True, CRC_EVERY_BYTE, CHANNEL_A_ONLY, False, False)
			Me.BusMaster.OW_ReleaseLock()

			'Me.BusMaster.incrementLogLevel()
			'Me.BusMaster.log("DS2406- Writing ChannelControlByte = " & Hex2(ChannelControlByte) & vbCrLf)
			'owResponse = Me.BusMaster.OW_WriteBlock(Hex2(CHANNEL_ACCESS_COMMAND) & Hex2(ChannelControlByte) & "FFFF", Me.ROMId)
			channelInfoByte = CByte("&h" & Mid(owResponse, 7, 2))
			'Me.BusMaster.log("DS2406- ChannelInfoByte Returned: " & Hex2(channelInfoByte) & vbCrLf)
			'Me.BusMaster.decrementLogLevel()
			Return channelInfoByte
		End Function
		Private Function fixJ(ByVal current_index As Integer, ByVal option_mask As Integer) As Integer
			'assume that current_index started at 0, but this function is never called at 0
			Select Case (option_mask And &H3)
				Case &H0
					Return 1				' no crc
				Case &H1
					Return 3				' 2-byte CRC after every byte
				Case Else			'must be 0x02 (after 8 bytes) or 0x03 (after 32 bytes)
					If ((current_index And (8 + (24 * (option_mask And &H1)) - 1)) = 0) Then
						Return 3
					End If
			End Select
			Return 1
		End Function

		Public Function computeCRC16(ByVal dataToCRC As Integer, ByVal seed As Integer) As Integer
			Dim dat As Integer
			Dim indx1 As Integer
			Dim indx2 As Integer

			dat = (((dataToCRC Xor (seed And &HFF))) And &HFF)
			seed = ((seed And &HFFFF) >> 8)

			indx1 = (dat And &HF)
			indx2 = (dat >> 4)

			If (ODD_PARITY(indx1) Xor ODD_PARITY(indx2)) = 1 Then
				seed = seed Xor &HC001
			End If

			dat = (dat << 6)
			seed = (seed Xor dat)
			dat = (dat << 1)
			seed = (seed Xor dat)

			Return seed
		End Function

		Public Function CRC16(ByVal dataToCrc As Integer) As Integer
			Return computeCRC16(dataToCrc, 0)
		End Function
		Public Function CRC16(ByVal dataToCrc As Integer, ByVal seed As Integer) As Integer
			Return computeCRC16(dataToCrc, seed)
		End Function

		Public Function CRC16(ByVal dataToCrc() As Byte) As Integer
			Return CRC16(dataToCrc, 0, dataToCrc.Length, 0)
		End Function

		Public Function CRC16(ByVal dataToCrc() As Byte, ByVal off As Integer, ByVal len As Integer, ByVal seed As Integer) As Integer
			'loop to do the crc on each data element
			Dim i As Integer
			For i = 0 To len - 1
				seed = computeCRC16(dataToCrc(i + off), seed)
			Next i

			Return seed

		End Function

		Public Function CRC16(ByVal dataToCrc() As Byte, ByVal seed As Integer) As Integer
			Return CRC16(dataToCrc, 0, dataToCrc.Length, seed)
		End Function
	End Class