
' ***************************************************************
' * \ \ 
' * Description:
' * The DS2423 1-Wire RAM with Counters device is a fully static,
' * read/write memory for battery operation in a low-cost,
' * six-lead TSOC, surface-mount package. The memory is organized
' * as 16 pages of 256 bits each. In addition, the device has
' * four counters, two of them with external trigger inputs
' * called A and B. Each of the counters is associated with a
' * memory page. A counter without external trigger input
' * increments each time data is written to the page it is
' * associated with (write cycle counter). The counters triggered
' * by inputs A and B, respectively, increment with every
' * low-going pulse on their input. All counters are readonly.
' * They are automatically cleared to 0 when the battery is
' * connected.
' * 
' * The battery-backed memory offers a simple solution to storing
' * and retrieving information pertaining to the equipment where
' * the DS2423 is installed and its frequency of use. The
' * scratchpad is an additional page that acts as a buffer when
' * writing to memory. Data is first written to the scratchpad
' * where it may be read back for verification. A copy scratchpad
' * command will then transfer the data to memory. This process
' * ensures data integrity when modifying the memory. A 64-bit
' * registration number is factory lasered into each DS2423 to
' * provide a guaranteed unique identity which allows for
' * absolute traceability and acts as node address if multiple
' * DS2423 are connected in parallel to form a local network.
' * Data is transferred serially via the 1-Wire protocol, which
' * requires only a single data lead and a ground return.        
' ***************************************************************
	Public Class DS2423
		Inherits Dallas1WireDevice

		Const PROTOCOL_WRITE_SCRATCHPAD As String = "0F"	 'DS2423 Function Command "WRITE SCRATCHPAD"
		Const PROTOCOL_READ_SCRATCHPAD As String = "AA"	 'DS2423 Function Command "READ SCRATCHPAD"
		Const PROTOCOL_COPY_SCRATCHPAD As String = "5A"	 'DS2423 Function Command "COPY SCRATCHPAD"
		Const PROTOCOL_READ_MEMORY As String = "F0"		'DS2423 Function Command "READ MEMORY"
		Const PROTOCOL_READ_MEMORY_PLUS_COUNTER As String = "A5"		'DS2423 Function Command "READ MEMORY + COUNTER"

		Public Sub New(ByVal ROMId As String, ByVal busMaster As OWInterface.BusMasterInterface)
			MyBase.New(ROMId, busMaster)
		End Sub
		' *******************************************************************************************************
		' * After issuing the Write Scratchpad command, the master must first provide the 2-byte target address,
		' * followed by the data to be written to the scratchpad. The data will be written to the scratchpad
		' * starting at the byte offset (T4:T0). The ending offset (E4: E0) will be the byte offset at which the
		' * master stops writing data. Only full data bytes are accepted. If the last data byte is incomplete its
		' * content will be ignored and the partial byte flag PF will be set. When executing the Write Scratchpad
		' * command the CRC generator inside the DS2423 (see Figure 12) calculates a CRC over the entire data
		' * stream, starting at the command code and ending at the last data byte sent by the master. This CRC is
		' * generated using the CRC16 polynomial by first clearing the CRC generator and then shifting in the
		' * command code (0FH) of the Write Scratchpad command, the Target Addresses TA1 and TA2 as supplied by
		' * the master and all the data bytes. The master may end the Write Scratchpad command at any time.
		' * However, if the ending offset is 11111b, the master may send 16 read time slots and will receive the
		' * CRC generated by the DS2423. The memory address range of the DS2423 is 0000H to 01FFH. If the bus
		' * master sends a target address higher than this, the internal circuitry of the chip will set seven
		' * most significant address bits to 0 as they are shifted into the internal address register. The Read
		' * Scratchpad command will reveal the target address as it will be used by the DS2423. The master will
		' * identify such address modifications by comparing the target address read back to the target address
		' * transmitted. If the master does not read the scratchpad, a subsequent Copy Scratchpad command will
		' * not work since the most significant bits of the target address the master sends will not match the
		' * value the DS2423 expects.                                                                            
		' *******************************************************************************************************
		Public Function WriteScratchpad(ByVal TA1 As Byte, ByVal TA2 As Byte, ByVal whatToWrite As Byte(), ByVal lockId As String)
			Dim localLock As Boolean
			Dim scratchPad As String
			Dim scratchPadDataLength As Integer

			'Use the 'Write Block' command to issue the write data to the scratchpad in the DS2423.
			'Since more than 28 bytes may be written, multiple write block commands may be issued
			'back to back in order to generate enough time slots to write all of the data to the DS2423.

			'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Write Scratchpad command, and send TA1, TA2
			Me.BusMaster.OW_WriteBlock(PROTOCOL_WRITE_SCRATCHPAD & Hex2(TA1) & Hex2(TA2), Me.ROMId)

			'The number of bytes remaining to be written to the scratchpad is determined by the length of the whatToWrite byte() parameter.
			'Now write the rest of the scratchpad data
			Dim packetSize As Integer
			Dim bytesToGo As Integer
			Dim bytesWritten As Integer
			bytesToGo = whatToWrite.Length()
			Do While bytesToGo > 0
				If bytesToGo > 28 Then
					'The write block command only supports 28 bytes per pass
					packetSize = 28
					bytesToGo = bytesToGo - 28
				Else
					packetSize = bytesToGo
					bytesToGo = 0
				End If
				For t As Integer = 0 To packetSize
					scratchPad = scratchPad & Hex2(whatToWrite(bytesWritten + t))
				Next
				Me.BusMaster.OW_WriteBlock(scratchPad)
				bytesWritten = bytesWritten + packetSize
			Loop

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If
		End Function

		' ******************************************************************************************************
		' * This command is used to verify scratchpad data and target address. After issuing the Read Scratchpad
		' * command, the master begins reading. The first 2 bytes will be the target address. The next byte will
		' * be the ending offset/data status byte (E/S) followed by the scratchpad data beginning at the byte
		' * \offset (T4: T0). The master may read data until the end of the scratchpad after which the data read
		' * will be all logic 1s.                                                                               
		' ******************************************************************************************************
		Public Function ReadScratchpad(ByVal lockId As String) As Byte()
			Dim localLock As Boolean
			Dim retVal As Byte()
			Dim scratchPad As String
			Dim scratchPadDataLength As Integer

			'Use the 'Write Block' command to issue the read scratchpad command to the DS2423.
			'Since more than 28 bytes will be read back, multiple write block commands must be issued
			'back to back in order to generate enough time slots to read all of the data from the DS2423.

			'Since multiple commands will be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Read Scratchpad command, and read TA1, TA2, Status E/S
			scratchPad = Me.BusMaster.OW_WriteBlock(PROTOCOL_READ_SCRATCHPAD & "FFFFFF", Me.ROMId).Data(0)
			scratchPad = Right$(scratchPad, Len(scratchPad) - 2)		 'Strips off the leading "Read Scratchpad" command byte.

			'The number of bytes remaining to be read from the scratchpad is determined by the contents
			'of T4:T0, from TA1, just read.
			scratchPadDataLength = 31 - ((CInt("&H" & Left$(scratchPad, 2))) And &H1F)

			'Now read the rest of the scratchpad data
			Dim packetSize As Integer
			Dim bytesToGo As Integer
			bytesToGo = scratchPadDataLength
			Do While bytesToGo > 0
				If bytesToGo > 28 Then
					'The write block command only supports 28 bytes per cycle
					packetSize = 28
					bytesToGo = bytesToGo - 28
				Else
					packetSize = bytesToGo
					bytesToGo = 0
				End If
				scratchPad = scratchPad & Me.BusMaster.OW_WriteBlock(StrDup(packetSize * 2, "F")).Data(0)
			Loop

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If

			'Convert the read data into a Byte()
			retVal = EmbeddedDataSystems.EDS_OWInterface.Hex2ByteArray(scratchPad)
			Return retVal

		End Function

		' ******************************************************************************************************
		' * This command is used to copy data from the scratchpad to memory. After issuing the Copy Scratchpad
		' * command, the master must provide a 3-byte authorization pattern which can be obtained by reading the
		' * scratchpad for verification. This pattern must exactly match the data contained in the three address
		' * registers (TA1, TA2, E/S, in that order). If the pattern matches, the AA (Authorization Accepted)
		' * flag will be set and the copy will begin. A pattern of alternating 1s and 0s will be transmitted
		' * after the data has been copied until a Reset Pulse is issued by the master. Any attempt to reset the
		' * part will be ignored while the copy is in progress. Copy typically takes 30s. The data to be copied
		' * is determined by the three address registers. The scratchpad data from the beginning offset through
		' * the ending offset will be copied to memory, starting at the target address. Anywhere from 1 to 32
		' * bytes may be copied to memory with this command. The AA flag will be cleared only by executing a
		' * Write Scratchpad command.                                                                           
		' ******************************************************************************************************
		Public Function CopyScratchpad(ByVal TA1 As Byte, ByVal TA2 As Byte, ByVal ES As Byte, ByVal lockid As String)
			Dim localLock As Boolean
			Dim retVal As Byte()
			Dim scratchPad As String
			Dim scratchPadDataLength As Integer

			'Use the 'Write Block' command to issue the copy scratchpad command to the DS2423.

			If IsNothing(lockid) Then
				'The bus isn't already locked, so acquire a lock.
				lockid = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Copy Scratchpad command, and xmit the authorization pattern TA1, TA2, E/S
			Me.BusMaster.OW_WriteBlock(PROTOCOL_COPY_SCRATCHPAD & Hex2(TA1) & Hex2(TA2) & Hex2(ES), Me.ROMId)
			System.Threading.Thread.Sleep(1)		 'Allow the part 1mS to copy the value to memory.

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If
		End Function

		' *******************************************************************************************************
		' * The read memory command may be used to read the entire memory. After issuing the command, the master
		' * must provide the 2-byte target address. After the two bytes, the master reads data beginning from the
		' * target address and may continue until the end of memory, at which point logic 1s will be read. It is
		' * important to realize that the target address registers will contain the address provided. The ending
		' * \offset/data status byte is unaffected. The hardware of the DS2423 provides a means to accomplish
		' * error-free writing to the memory section. To safeguard reading data in the 1-Wire environment and to
		' * simultaneously speed up data transfers, it is recommended to packetize data into data packets of the
		' * size of one memory page each. Such a packet would typically store a 16-bit CRC with each page of data
		' * to ensure rapid, error-free data transfers that eliminate having to read a page multiple times to
		' * determine if the received data is correct or not. (See the Book of DS19xx iButton Standards, Chapter
		' * 7 for the recommended file structure.)                                                               
		' *******************************************************************************************************
		Public Function ReadMemory(ByVal TA1 As Byte, ByVal TA2 As Byte, ByVal numBytes As Integer, ByVal lockid As String) As Byte()
			Dim localLock As Boolean
			Dim retVal As Byte()
			Dim scratchPad As String
			Dim scratchPadDataLength As Integer

			'Use the 'Write Block' command to issue the read memory command to the DS2423.
			'Since more than 28 bytes might be read back, multiple write block commands may be issued
			'back to back in order to generate enough time slots to read all of the data from the DS2423.

			'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockid) Then
				'The bus isn't already locked, so acquire a lock.
				lockid = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Read memory command, and transmit TA1, TA2
			Me.BusMaster.OW_WriteBlock(PROTOCOL_READ_MEMORY & Hex2(TA1) & Hex2(TA2), Me.ROMId)

			'The number of bytes to be read from the device is determined by the numBytes parameter
			Dim packetSize As Integer
			Dim bytesToGo As Integer
			bytesToGo = numBytes
			Do While bytesToGo > 0
				If bytesToGo > 28 Then
					'The write block command only supports 28 bytes per cycle
					packetSize = 28
					bytesToGo = bytesToGo - 28
				Else
					packetSize = bytesToGo
					bytesToGo = 0
				End If
				scratchPad = scratchPad & Me.BusMaster.OW_WriteBlock(StrDup(packetSize * 2, "F")).Data(0)
			Loop

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If

			'Convert the read data into a Byte()
			retVal = EmbeddedDataSystems.EDS_OWInterface.Hex2ByteArray(scratchPad)
			Return retVal
		End Function

		' *******************************************************************************************************
		' * The Read Memory + Counter command is used to read memory data together with the write cycle counter
		' * \or externally triggered counter associated with the addressed page of data memory. The additional
		' * information is transmitted by the DS2423 as the end of a memory page is encountered. Following the
		' * current value of the counter the DS2423 transmits 32 0-bits and a 16-bit CRC generated by the DS2423.
		' * After having sent the command code of the Read Memory + Counter command, the bus master sends a
		' * two-byte address (TA1 = (T7:T0), TA2 = (T15:T8)) that indicates a starting byte location within the
		' * data field. With the subsequent read data time slots the master receives data from the DS2423
		' * starting at the initial address and continuing until the end of a 32-byte page is reached. At that
		' * point the bus master will send 80 additional read data time slots and receive the contents of the
		' * 32-bit counter associated with the addressed page, 32 0-bits and a 16-bit CRC. With subsequent read
		' * data time slots the master will receive data starting at the beginning of the next page followed by
		' * the contents of the counter associated with the page, 0-bits and CRC for that page. This sequence
		' * will continue until the final page and its accompanying data is read by the bus master. When applying
		' * the Read Memory + Counter command to a page that does not have a counter associated, the master will
		' * read FFFFFFFFH instead of a valid count. With the initial pass through the Read Memory + Counter flow
		' * chart the 16-bit CRC value is the result of shifting the command byte into the cleared CRC generator,
		' * followed by the two address bytes, the contents of the data memory, the counter and the 0-bits.
		' * Subsequent passes through the Read Memory + Counter flow chart will generate a 16-bit CRC that is the
		' * \result of clearing the CRC generator and then shifting in the contents of the data memory page, its
		' * associated counter and 0-bits. After the 16-bit CRC of the last page is read, the bus master will
		' * receive logical 1s from the DS2423 until a Reset Pulse is issued. The Read Memory + Counter command
		' * sequence can be ended at any point by issuing a Reset Pulse.                                         
		' *******************************************************************************************************
		Public Function ReadMemoryPlusCounter(ByVal TA1 As Byte, ByVal TA2 As Byte, ByVal lockid As String) As Byte()
			Dim localLock As Boolean
			Dim retVal As Byte()
			Dim numBytes As Integer
			Dim memoryPage As String

			'Use the 'Write Block' command to issue the read memory + Counter command to the DS2423.
			'Since more than 28 bytes might be read back, multiple write block commands may be issued
			'back to back in order to generate enough time slots to read all of the data from the DS2423.

			'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockid) Then
				'The bus isn't already locked, so acquire a lock.
				lockid = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Read Memory + Counter command, and transmit TA1, TA2
			Me.BusMaster.OW_WriteBlock(PROTOCOL_READ_MEMORY_PLUS_COUNTER & Hex2(TA1) & Hex2(TA2), Me.ROMId)

			'The number of bytes to be read from the device is determined by:
			'Reading from the starting address to the end of a 32 byte boundary
			'Then 10 additional bytes as follows:
			'	32 bit counter
			'	32 zero bits
			'	16 bit CRC
			numBytes = (32 - (TA2 Mod 32)) + 10

			Dim packetSize As Integer
			Dim bytesToGo As Integer
			bytesToGo = numBytes
			Do While bytesToGo > 0
				If bytesToGo > 28 Then
					'The write block command only supports 28 bytes per cycle
					packetSize = 28
					bytesToGo = bytesToGo - 28
				Else
					packetSize = bytesToGo
					bytesToGo = 0
				End If
				memoryPage = memoryPage & Me.BusMaster.OW_WriteBlock(StrDup(packetSize * 2, "F")).Data(0)
			Loop

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If

			'Convert the read data into a Byte()
			retVal = EmbeddedDataSystems.EDS_OWInterface.Hex2ByteArray(memoryPage)
			Return retVal

		End Function

	End Class