
' ***************************************************************
' * Description:
' * The DS2438 Smart Battery Monitor provides several functions
' * that are desirable to carry in a battery pack: a means of
' * tagging a battery pack with a unique serial number, a
' * direct-to-digital temperature sensor which eliminates the
' * need for thermistors in the battery pack, an A/D converter
' * which measures the battery voltage and current, an integrated
' * current accumulator which keeps a running total of all
' * current going into and out of the battery, an elapsed time
' * meter, and 40 bytes of nonvolatile EEPROM memory for storage
' * \of important parameters such as battery chemistry, battery
' * capacity, charging methodology and assembly date. Information
' * is sent to/from the DS2438 over a 1-Wire interface, so that
' * \only one wire (and ground) needs to be connected from a
' * central microprocessor to a DS2438. This means that battery
' * packs need only have three output connectors: battery power,
' * ground, and the 1-Wire interface.
' * 
' * Because each DS2438 contains a unique silicon serial number,
' * multiple DS2438s can exist on the same 1-Wire bus. This
' * allows multiple battery packs to be charged or used in the
' * system simultaneously. Applications for the smart battery
' * monitor include portable computers, portable/cellular
' * telephones, and handheld instrumentation battery packs in
' * which it is critical to monitor real-time battery
' * performance. Used in conjunction with a microcontroller in
' * the host system, the DS2438 provides a complete smart battery
' * pack solution that is fully chemistry-independent. The
' * customization for a particular battery chemistry and capacity
' * is realized in the code programmed into the microcontroller
' * and DS2438 EEPROM, and only a software revision is necessary
' * should a designer wish to change battery pack chemistry.     
' ***************************************************************
	Public Class DS2438
		Inherits Dallas1WireDevice

		Const PROTOCOL_WRITE_SCRATCHPAD As String = "4E"	 'DS2438 Function Command "WRITE SCRATCHPAD"
		Const PROTOCOL_READ_SCRATCHPAD As String = "BE"	 'DS2438 Function Command "READ SCRATCHPAD"
		Const PROTOCOL_COPY_SCRATCHPAD As String = "48"	 'DS2438 Function Command "COPY SCRATCHPAD"
		Const PROTOCOL_RECALL_MEMORY As String = "B8"		'DS2438 Function Command "RECALL MEMORY"
		Const PROTOCOL_CONVERT_T As String = "44"		'DS2438 Function Command "CONVERT T"
		Const PROTOCOL_CONVERT_V As String = "B4"		'DS2438 Function Command "CONVERT V"

		Public Const AD_SOURCE_VDD = 1
		Public Const AD_SOURCE_VAD = 0

		'DS2438 Memory Map
		'===============================================================================================================
		'Page 0 (00h)
		'The first page contains the most frequently accessed information of the DS2438, and most locations are
		'volatile read-only bytes with the exception of the Status/Configuration Register (Byte 0) and the
		'Threshold Register (Byte 7). The Status/Configuration Register is a nonvolatile read/write byte which
		'defines which features of the DS2438 are enabled and how they will function. The register is formatted as
		'follows:

		'IAD = Current A/D Control Bit. 1 = the current A/D and the ICA are enabled, and current
		'measurements will be taken at the rate of 36.41 Hz; 0 = the current A/D and the ICA have been
		'disabled. The default value of this bit is a 1 (current A/D and ICA are enabled).
		Const CONFIG_IAD As Byte = 1	 'Current A/D Control Bit

		'CA = Current Accumulator Configuration. 1 = CCA/DCA is enabled, and data will be stored and can
		'be retrieved from page 7, bytes 4-7; 0 = CCA/DCA is disabled, and page 7 can be used for general
		'EEPROM storage. The default value of this bit is a 1 (current CCA/DCA are enabled).
		Const CONFIG_CA As Byte = 2	 'Current Accumulator Configuration

		'EE = Current Accumulator Shadow Selector bit. 1 = CCA/DCA counter data will be shadowed to
		'EEPROM each time the respective register is incremented; 0= CCA/DCA counter data will not be
		'shadowed to EEPROM. The CCA/DCA could be lost as the battery pack becomes discharged. If the CA
		'bit in the status/configuration register is set to 0, the EE bit will have no effect on the DS2438
		'functionality. The default value of this bit is a 1 (current CCA/DCA data shadowed to EEPROM).
		Const CONFIG_EE As Byte = 4	 'Current Accumulator Shadow Selector Bit

		'AD = Voltage A/D Input Select Bit. 1 = the battery input (VDD) is selected as the input for the
		'DS2438 voltage A/D converter; 0 = the general purpose A/D input (VAD) is selected as the voltage
		'A/D input. For either setting, a Convert V command will initialize a voltage A/D conversion. The default
		'value of this bit is a 1 (VDD is the input to the A/D converter).
		Const CONFIG_AD As Byte = 8	 'Voltage A/D Input Select Bit

		'TB = Temperature Busy Flag. 1 = temperature conversion in progress; 0 = temperature conversion
		'complete.
		Const CONFIG_TB As Byte = 16	 'Temperature Busy Flag

		'NVB = Nonvolatile Memory Busy Flag. 1 = Copy from Scratchpad to EEPROM in progress; 0 =
		'Nonvolatile memory not busy. A copy to EEPROM may take from 2 ms to 10 ms (taking longer at lower
		'supply voltages).
		Const CONFIG_NVB As Byte = 32	 'Nonvolatile Memory Busy Flag

		'ADB = A/D Converter Busy Flag. 1 = A/D conversion in progress on battery voltage; 0 = conversion
		'complete, or no measurement being made. An A/D conversion takes approximately 10 ms.
		Const CONFIG_ADB As Byte = 64	 'A/D Converter Busy Flag

		'X = Dont care
		Const CONFIG_X As Byte = 128		'Don't care

		'Bytes 1 and 2 of page 0 contain the last completed temperature conversion in the format described in the
		'Operation - Measuring Temperature section.

		' Bytes 3-4 contain the last completed voltage A/D
		'conversion result and bytes 5-6 contain the instantaneous current data.

		' Byte 7 contains the Threshold Register. Refer to the appropriate section for the data format of these locations.

		'NOTE:
		'The data in the scratchpad of the status and threshold register will determine the operation of the device.

		'===============================================================================================================
		'Page 1 (01h)
		'The second page, Page 1, contains the ICA, elasped time meter, and current offset data. Both the ETM
		'and ICA are volatile read/write locations so that they may be set, changed, or cleared by the host
		'software. Bytes 0-3 contain the ETM data, formatted as described in the Operation - Elapsed Time
		'Meter section. Byte 4 contains the 8-bit ICA. Bytes 5 and 6 contain the Offset Register data. Byte 7 is
		'reserved and will read out as all 1s.

		'===============================================================================================================
		'Page 2 (02h)
		'The third page of memory (Page 2) contains the Disconnect (first 4 bytes) and End of Charge (remaining
		'4 bytes) timestamps. This page is volatile and read/write. Refer to the Operation  Elapsed Time Meter
		'section for the formatting of these locations.
		'===============================================================================================================
		'Pages 3-7 (03h - 07h)
		'The remainder of the memory in the DS2438 (Pages 3 through 7) is backed with EEPROM. This memory
		'provides 40 bytes of user memory which may be used to carry any information the user wishes to store.
		'Additionally, the CCA/DCA information is stored in bytes 4-7 of page 7 if the DS2438 is configured
		'appropriately. If the CCA/DCA is used, page 7 should not be written to or current accumulator data will
		'be overwritten. See Operation-Current Accumulators for details.
		'===============================================================================================================

		Public Sub New(ByVal ROMId As String, ByVal busMaster As OWInterface.BusMasterInterface)
			MyBase.New(ROMId, busMaster)
		End Sub

		Public Function GetTemperature(ByVal lockid As String) As Single
			Dim localLock As Boolean
			Dim scratchPad As Byte()
			Dim tempRegister As Long
			Dim sign As Integer
			Dim retVal As Single

			'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockid) Then
				'The bus isn't already locked, so acquire a lock.
				lockid = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Instruct the DS2438 to perform the temperature conversion
			Me.Convert_T(lockid)
			'Recall memory page 0 to the scratchpad
			Me.RecallMemory(0, lockid)

			'Now read back memory page 0, which contains the conversion results
			'in byte numbers 1 and 2
			scratchPad = Me.ReadScratchpad(0, lockid)

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If

			'Parse the returned temperature (expressed in .03125 C units)
			tempRegister = ((scratchPad(2) * 256) Or scratchPad(1))
			If (tempRegister And &H8000) Then
				'Sign bit is set, temperature is negative
				'Perform Twos compliment conversion
				sign = -1
				tempRegister = (((Not tempRegister) + 1) And &HFFFF)
			Else
				'Sign bit is not set, temperature is not negative
				sign = 1
			End If

			tempRegister = tempRegister >> 3		 'Remove 3 LSbits
			retVal = tempRegister * 0.03125 * sign		 'In Celcius

			Return retVal
		End Function

		' ************************************************************************
		' * Initiates a voltage conversion and returns the converted
		' * value expressed in Volts.
		' * Parameters:
		' * ADSelect :  Configure the AD bit in the Status/Configuration Register
		' *             (byte 0) of memory Page 0 to select the input on which to
		' *             perform the A/D conversion.<p/>Possible values for the A/D
		' *             bit are\:<p/> 1 = The battery input (VDD)<p/> 0 = General
		' *             purpose A/D input (VAD)
		' * lockid :    Previously aquired lock\-id, if the bus is already locked
		' *             and being managed by the caller. If = Nothing, then a lock
		' *             will be acquired and released locally.                    
		' ************************************************************************
		Public Function GetVoltage(ByVal ADSelect As Byte, ByVal lockid As String) As Single

			Dim configByte As Byte
			Dim localLock As Boolean
			Dim scratchPad As Byte()
			Dim milliVolts_10 As Long
			Dim retVal As Single

			'Configure the AD bit in the Status/Configuration Register (byte 0) of memory Page 0 to select
			'the input on which to perform the A/D conversion.
			'Possible values for the A/D bit are:
			'1 = The battery input (VDD)
			'0 = General purpose A/D input (VAD)

			'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockid) Then
				'The bus isn't already locked, so acquire a lock.
				lockid = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Start by reading the existing configuration byte
			scratchPad = Me.ReadScratchpad(0, lockid)
			configByte = (scratchPad(0) Or CONFIG_AD)
			'Now set the desired input source
			configByte = (configByte And (ADSelect * CONFIG_AD))
			scratchPad(0) = configByte

			'Write the new configuration byte back to the DS2438
			Me.WriteScratchpad(0, scratchPad, lockid)
			'Me.CopyScratchpad(0, lockid)

			'Now perform the Voltage conversion
			Me.Convert_V(lockid)

			'Recall memory page 0 to the scratchpad
			Me.RecallMemory(0, lockid)

			'Now read back memory page 0, which contains the conversion results
			'in byte numbers 3 and 4
			scratchPad = Me.ReadScratchpad(0, lockid)
			'Parse the returned voltage (expressed in 10 mV units)
			milliVolts_10 = ((scratchPad(4) * 256) Or scratchPad(3))

			'Convert the parsed value to Volts
			retVal = milliVolts_10 / 100.0

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If

			Return retVal
		End Function

		' ***************************************************************
		' * This command writes to the scratchpad page xx of the DS2438.
		' * The entire 8-byte scratchpad space may be written, but all
		' * writing begins with the byte present at address 0 of the
		' * selected scratchpad. After issuing this command, the user
		' * must send the page number of the scratchpad to be written;
		' * then the user may begin writing data to the DS2438
		' * scratchpad. Writing may be terminated at any point by issuing
		' * a reset. Valid page numbers for writing are 00h-07h.         
		' ***************************************************************
		Public Function WriteScratchpad(ByVal PageNumber As Byte, ByVal whatToWrite As Byte(), ByVal lockId As String)
			Dim localLock As Boolean
			Dim scratchPad As String
			Dim scratchPadDataLength As Integer

			'Use the 'Write Block' command to issue the write data to the scratchpad in the DS2438.

			'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Write Scratchpad command, and send the page number followed by the data to be written to the scratchpad.
			Me.BusMaster.OW_WriteBlock(PROTOCOL_WRITE_SCRATCHPAD & Hex2(PageNumber) & Hex2(whatToWrite), Me.ROMId)

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If
		End Function

		' ***************************************************************
		' * This command reads the contents of the scratchpad page xxh on
		' * the DS2438. After issuing this command, the user must send
		' * the page number of the scratchpad to be read, and then may
		' * begin reading the data, always beginning at address 0 of the
		' * selected scratchpad. The user may read through the end of the
		' * scratchpad space (byte 07h), with any reserved data bits
		' * reading all logic 1s, then read the CRC of the data, and
		' * after which the data read will be all logic 1s. If not all
		' * locations are to be read, the master may issue a reset to
		' * terminate reading at any time. Valid page numbers are 00h -
		' * 07h.                                                         
		' ***************************************************************
		Public Function ReadScratchpad(ByVal pageNumber As Byte, ByVal lockId As String) As Byte()
			Dim localLock As Boolean
			Dim retVal As Byte()
			Dim scratchPad As String

			'Use the 'Write Block' command to issue the read scratchpad command to the DS2428.

			'Since multiple commands will be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Read Scratchpad command, followed by the page number,  and then read 8 data bytes
			scratchPad = Me.BusMaster.OW_WriteBlock(PROTOCOL_READ_SCRATCHPAD & Hex2(pageNumber) & "FFFFFFFFFFFFFFFF", Me.ROMId).Data(0)
			scratchPad = Right$(scratchPad, Len(scratchPad) - 4)			'Strips off the leading "Read Scratchpad" command byte and Page Number byte.

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If

			'Convert the read data into a Byte()
			retVal = EmbeddedDataSystems.EDS_OWInterface.Hex2ByteArray(scratchPad)
			Return retVal

		End Function

		' ***************************************************************
		' * This command copies the scratchpad page xx into the EEPROM /
		' * SRAM memory page xxh of the DS2438. After issuing this
		' * command, the user must write a page number to direct which
		' * page of memory the scratchpad is to be copied. Valid page
		' * numbers are 00h - 07h. During the copy function, the NVB bit
		' * in the Status/configuration register will be set to a l.
		' * When the copy is complete, this bit will reset to 0. If the
		' * bus master issues read time slots following this command, the
		' * DS2438 will output 0 on the bus as long as it is busy
		' * copying the scratchpad to SRAM/EEPROM; it will return a 1
		' * when the copy process is complete.                           
		' ***************************************************************
		Public Function CopyScratchpad(ByVal pageNumber As Byte, ByVal lockId As String)
			Dim localLock As Boolean

			'Use the 'Write Block' command to issue the copy scratchpad command to the DS2428.

			'Since multiple commands will be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Copy Scratchpad command
			Me.BusMaster.OW_WriteBlock(PROTOCOL_COPY_SCRATCHPAD & Hex2(pageNumber), Me.ROMId)

			Do While Me.BusMaster.OW_WriteBit(1).Data(0) = 0
				'Wait for the Copy Scratchpad command to complete
				'TODO- Add some timeout here for sanity.
			Loop

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If
		End Function

		' **************************************************************
		' * This command recalls the stored values in EEPROM / SRAM page
		' * \xxh to the scratchpad page xxh. This command must proceed a
		' * Read SPxx command in order to read any page of memory on the
		' * DS2438. Valid page numbers are 00h - 07h.                   
		' **************************************************************
		Public Function RecallMemory(ByVal pageNumber As Byte, ByVal lockId As String)
			Dim localLock As Boolean

			'Use the 'Write Block' command to issue the copy scratchpad command to the DS2428.

			'Since multiple commands will be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Recall Memory command followed by the page number to be recalled.
			Me.BusMaster.OW_WriteBlock(PROTOCOL_RECALL_MEMORY & Hex2(pageNumber), Me.ROMId)

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If
		End Function

		' ***************************************************************
		' * This command begins a temperature conversion. No further data
		' * is required. The temperature conversion will be performed,
		' * setting the TB flag in the Status/Configuration register to a
		' * 1 during conversion. When the temperature conversion is
		' * done, the TB flag will clear to a 0. If the bus master
		' * issues read time slots following this command, the DS2438
		' * will output 0 on the bus as long as it is busy making a
		' * temperature conversion; it will return a 1 when the
		' * temperature conversion is complete.                          
		' ***************************************************************
		Public Function Convert_T(ByVal lockId As String)
			Dim localLock As Boolean

			'Use the 'Write Block' command to issue the convert_T command to the DS2428.

			'Since multiple commands will be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Convert_T command.
			Me.BusMaster.OW_WriteBlock(PROTOCOL_CONVERT_T, Me.ROMId)
			Do While Me.BusMaster.OW_WriteBit(1).Data(0) = 0
				'Wait for the conversion to complete
				'TODO- Add some timeout here for sanity.
			Loop

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If
		End Function

		' **************************************************************
		' * This command instructs the DS2438 to initiate a voltage
		' * analog-to-digital conversion cycle. This sets the ADB flag
		' * (see Status/Configuration register discussion in the Memory
		' * Map section). The voltage supply that is measured is defined
		' * by the AD bit of the Status/Configuration register. When the
		' * A/D conversion is done, the ADB flag is cleared and the
		' * current voltage value is placed in the VOLTAGE REGISTER of
		' * page 00h. While an A/D conversion is taking place, all other
		' * memory functions are still available for use. If the bus
		' * master issues read time slots following this command, the
		' * DS2438 will output 0 on the bus as long as it is busy
		' * making a voltage measurement; it will return a 1 when the
		' * conversion is complete.                                     
		' **************************************************************
		Public Function Convert_V(ByVal lockId As String)
			Dim localLock As Boolean

			'Use the 'Write Block' command to issue the convert_V command to the DS2428.

			'Since multiple commands will be issued to the 1-Wire bus to accomplish this function, it is
			'neccessary to lock the bus to prevent interruption.
			If IsNothing(lockId) Then
				'The bus isn't already locked, so acquire a lock.
				lockId = Me.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			'Issue the Convert_V command.
			Me.BusMaster.OW_WriteBlock(PROTOCOL_CONVERT_V, Me.ROMId)
			System.Threading.Thread.Sleep(10)
			'Do While Me.BusMaster.OW_WriteBit(1).Data(0) = 0
			'	'Wait for the conversion to complete
			'	'TODO- Add some timeout here for sanity.
			'Loop

			'Reset the bus
			Me.BusMaster.OW_ResetBus()

			'Release the bus lock, if managed locally
			If localLock Then
				Me.BusMaster.OW_ReleaseLock()
			End If
		End Function

	End Class