
' ********************************************************************
' * The Embedded Data Systems' analog output class represents a
' * family of devices that contain an EDS 12-bit analog output
' * module that has been outfitted with a Dallas Semiconductor
' * 1-wire front-end. By implementing this configuration,
' * Embedded Data Systems is able to put a 1-Wire front end on a
' * wide variety of output devices that would not otherwise be
' * accessible to the 1-Wire bus. In this fashion, you only have
' * to implement the interface once in your software in order to
' * communicate to a wide variety of analog output devices.
' * 
' * Embedded Data Systems' IO devices that currently support this
' * interface are as follows:
' * 
' * 
' * 
' *     * Z2Ten -Zero to Ten Volt Output Module
' * 
' * 
' * 
' * The EDS analog outputs incorporate a DS2406 Addressable
' * Switch which sits on the 1-Wire bus and provides a 1-Wire
' * gateway to the 12-bit analog subsystem. The DS2406 is also
' * used to provide each device with unique identification, and
' * storage of calibration information and other usage
' * information.
' * 
' * Unique device identification and calibration information is
' * stored in the OTP memory of the DS2406. This information can
' * include, among other things:
' * 
' *     * Device ID - Used to determine the type of device
' *     * PollTime - Desired Polltime
' *     * LabelName - Label name of the device Ex.: "Z2TEN"
' *     * Units Label - Units label Ex.: "Volts"
' *     * Calibration Table - Provides enough data to allow the
' *       application to convert from the raw binary units returned by
' *       the analog subsystem to engineering units.
' * 
' * All of this data is written to the DS2406's OTP memory during
' * the manufacturing process of the device.
' * 
' * General usage is as follows:
' * 
' * To set the analog output:
' * 
' * The DS2406 is addressed and then setup to manipulate the
' * PIO_A pin. Data is then written to the digital-to-analog
' * converter using PIO_A. The value sent to the analog output is
' * a 16 bit signed binary value (2's compliment).
' * 
' * To read the analog output:
' * 
' * The DS2406 is addressed and then setup to manipulate the
' * PIO_A channel in order to read information from the
' * analog-to-digital converter. The value received from the
' * analog output is a 16 bit signed binary value (2's
' * compliment)
' * 
' * The bus master can use the calibration information contained
' * in the OTP to convert the 12-bit analog reading to a
' * calibrated engineering unit.                                      
' ********************************************************************
	Public Class EDSAnalogOutput

		Public PrimaryDS2406 As DS2406	 'Reference to the DS2406 acting as a gateway to the analog subsystem on this device. 
		Private pollTime As Integer		' Recommended polling rate for this device.
		Private labelName As String	 ' Name of the IO device type. Ex.: "Z2TEN"
		Private unitsLabel As String	 ' Name of the engineering units used on this device. Ex.:  "Volts"                                                
		Private reservedLabel As String	 ' Reserved for future use.
		Private engrScale As Decimal		' Engineering value when analog system is at full scale.
		Private engrOffset As Decimal		' Minimum engineering value.
		Private lastEngineeringValue As Double	 'Engineering value obtained during last readValue()

		Private ds2406MemoryPages As ArrayList	 'Local cache of the DS2406 memory contents...used only for generating documentation.
		Private creatingDocumentation As Boolean = False		'Indicates that this object should log all operations for the purpose of creating documentation.
		Private docs As String	 'Storage for generated documentation


		' **************************************************************
		' * Summary:
		' * Default constructor for EDSAnalogOutput
		' * 
		' * Description:
		' * Requires passing in a reference to the primary DS2406 that
		' * acts as the gateway to the A/D subsystem on the device. This
		' * DS2406 might have been previously discovered using
		' * EDSAnalogOutput.getEDSAnalogProbeDeviceType().
		' * 
		' * After noting the reference to the controlling DS2406, a call
		' * is made to initialzeAnalogOutput(), which reads all of the
		' * configuration data out of the DS2406 pertaining to this
		' * analog output device.                                       
		' **************************************************************
		Public Sub New(ByVal primaryDS2406 As DS2406)
			Me.PrimaryDS2406 = primaryDS2406
			initializeAnalogOutput()
		End Sub

		' *******************************************************
		' * Summary:
		' * Reads identification and scaling information from the
		' * controlling DS2406.
		' * 
		' * See Also:
		' * parseConfigurationData()                             
		' *******************************************************
		Private Sub initializeAnalogOutput()
			Const pageLength As Byte = 32
			Dim memoryPages As ArrayList
			Dim memoryPage As Char()
			Dim memoryBuffer(128) As Char


			Me.log("Initializing the AnalogOutput Device.")
			Me.log("This process reads the identification and scaling information from the controlling DS2406.")

			'Read the configuration data out of the DS2406's memory pages
			'and store into a single continuous char()
			Me.log("Start by reading all 4 memory pages from the onboard DS2406.")
			Me.PrimaryDS2406.BusMaster.clearLog()
			memoryPages = Me.PrimaryDS2406.ReadMemoryPages(0, 4)
			Me.log(Me.PrimaryDS2406.BusMaster.getLog())
			Me.ds2406MemoryPages = memoryPages
			For t As Integer = 0 To 3
				Me.log("Contents of memory page #" & CStr(t + 1) & ": " & memoryPages(t))
				memoryPage = EDS_OWInterface.Hex2CharArray(memoryPages(t))
				For position As Integer = 0 To 31
					memoryBuffer((t * 32) + position) = memoryPage(position)
				Next
			Next

			'Now parse the probe's configuration data out the the char()
			parseConfigurationData(memoryBuffer)
		End Sub

		' **************************************************************
		' * Summary:
		' * The EDS Analog Output devices also contain the ability to
		' * read the active output levels. This allows for closed-loop
		' * monitoring. By reading the output level, you can verify that
		' * the output device was able to drive its load to the intended
		' * level. This function demonstrates how to read the current
		' * analog value from the device.
		' * 
		' * Returns:
		' * The current output level of the analog output device. The
		' * returned value is expressed as a decimal in engineering
		' * units.                                                      
		' **************************************************************
		Public Function readValue() As Decimal
			Dim owresults As String
			Dim byteResults As Char()
			Dim analogValue As Integer
			Dim engineeringValue As Decimal
			Dim scale As Decimal
			Dim offset As Decimal
			Dim readAnalog As Byte() = New Byte(3) {&HFB, &HFF, &HFF, &HFF}

			Me.log("Read the current analog value...")
			Me.log("The " & Me.labelName & " device also supports the ability to read the active output levels. This allows for closed-loop monitoring. By reading the output level, you can verify that the output device was able to drive its load to the intended level. This demonstrates how to read the current analog value from the device.")
			Me.log("")
			Me.log("To read the value of the analog output on the " & Me.labelName & " , 6 data bytes need to be written out the PIO_A pin ")
			Me.log("on the DS2406.  The six bytes are constant: 0xFB, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF.")

			Me.log("To write this data out of the DS2406 PIO_A pin, configure the DS2406 for Toggle Read/Writing, Write Initially, CRC Every 8 Bytes, Channel A Access Only, No Interleave")
			Me.log("This is accomplished by transmitting the bytes 0xF5, 0xA6, 0xFF, 0xFF, 0xFF to the DS2406 immediately prior to the bytes you want to write to the PIO_A pin.")

			Me.PrimaryDS2406.BusMaster.clearLog()
			Me.PrimaryDS2406.BusMaster.OW_GetLock()
			Me.PrimaryDS2406.BusMaster.OW_AddressDevice(Me.PrimaryDS2406.ROMId)
			owresults = Me.PrimaryDS2406.ChannelAccess(readAnalog, True, False, DS2406.CRC_EVERY_8_BYTES, DS2406.CHANNEL_A_ONLY, True, False)
			Me.PrimaryDS2406.BusMaster.OW_ReleaseLock()

			Me.log("Putting this all together, results in the following bus master communications:")
			Me.log(Me.PrimaryDS2406.BusMaster.getLog())

			byteResults = EDS_OWInterface.Hex2CharArray(owresults)
			Me.log("")
			Me.log("Now parse the value out of the returned data...")
			Me.log("The MSB is located at byte 12, and the LSB is located at byte 10.")
			Me.log("Hex analog value: " & "0x" & Hex2(AscW(byteResults(11))) & Hex2(AscW(byteResults(9))))
			analogValue = CDec("&h" & Hex2(AscW(byteResults(11))) & Hex2(AscW(byteResults(9))))
			Me.log("Decimal analog value: " & CStr(analogValue))

			Me.log("Now convert from analog units to engineering units using the following formuala:")
			Me.log("(EngineeringScale / 32767) * AnalogValue + EngineeringOffset")
			Me.log("(" & CStr(engrScale) & " / 32767) * " & CStr(analogValue) & " + " & CStr(engrOffset))
			engineeringValue = (Me.engrScale / 32767) * analogValue + Me.engrOffset

			Me.lastEngineeringValue = engineeringValue
			Me.log("Engineering Value: " & CStr(engineeringValue) & " " & Me.unitsLabel)
			Return engineeringValue
		End Function

		' ***********************************************************
		' * Summary:
		' * This function demonstrates how to set the output of the
		' * analog output to a known value.
		' * 
		' * Parameters:
		' * newEngineeringValue :  The new output level, specified in
		' *                        engineering units.                
		' ***********************************************************
		Public Sub setValue(ByVal newEngineeringValue As Decimal)
			Dim owresults As String
			Dim analogValue As Integer
			Dim setAnalog As Byte() = New Byte(6) {&H5, &HFF, &HFF, &HFF, 0, 0, 0}

			Me.log("Set output level to " & CStr(newEngineeringValue) & " " & Me.unitsLabel)

			Me.log("Calculating new analog level using the following formula:")
			Me.log("analogValue = (EngineeringValue / (Engineering Scale / 32767)) + Engineering Offset")
			Me.log("analogValue = (" & CStr(newEngineeringValue) & " / (" & CStr(Me.engrScale) & " / 32767)) + " & CStr(Me.engrOffset))
			analogValue = (newEngineeringValue / (Me.engrScale / 32767)) + Me.engrOffset
			'analogValue = 32766
			Me.log("Analog Value = " & CStr(analogValue))

			Me.log("To change the value of the analog output on the " & Me.labelName & " , 9 data bytes need to be written out the PIO_A pin ")
			Me.log("on the DS2406.  The first four bytes are constant: 0x05, 0xFF, 0xFF, 0xFF.")
			Me.log("The next 3 bytes are as follows:")
			setAnalog(4) = analogValue And &HFF			'Sets the Low Byte of analog out
			setAnalog(5) = (analogValue >> 8)		 'Sets the high byte of analog out
			setAnalog(6) = setAnalog(4) Xor setAnalog(5)			'Sets the Error Check: LowByte XOR High byte
			Me.log("Byte 5: The LSB of the analog value calculated above.  0x" & Hex2(setAnalog(4)) & " for this example.")
			Me.log("Byte 6: The MSB of the analog value calculated above.  0x" & Hex2(setAnalog(5)) & " for this example.")
			Me.log("Byte 7: An error check byte calculated by (LSB XOR MSB). 0x" & Hex2(setAnalog(6)) & " for this example.")

			Me.log("The last two bytes are constant, 0xFF, 0xFF.  These are used to generate the 1-Wire time slots neccessary to read back CRC of the communications.")

			Me.log("To write this data out of the DS2406 PIO_A pin, configure the DS2406 for Write Only, Write Initially, CRC Every 8 Bytes, Channel A Access Only, No Interleave")
			Me.log("This is accomplished by transmitting the bytes 0xF5, 0x86, 0xFF, 0xFF, 0xFF to the DS2406 immediately prior to the bytes you want to write to the PIO_A pin.")

			Me.PrimaryDS2406.BusMaster.clearLog()
			Me.PrimaryDS2406.BusMaster.OW_GetLock()
			Me.PrimaryDS2406.BusMaster.OW_AddressDevice(Me.PrimaryDS2406.ROMId)
			owresults = Me.PrimaryDS2406.ChannelAccess(setAnalog, False, False, DS2406.CRC_EVERY_8_BYTES, DS2406.CHANNEL_A_ONLY, True, False)
			Me.PrimaryDS2406.BusMaster.OW_ResetBus()
			Me.PrimaryDS2406.BusMaster.OW_ReleaseLock()

			Me.log("Putting this all together results in the following bus master communications:")
			Me.log(Me.PrimaryDS2406.BusMaster.getLog())

		End Sub

		' *************************************************************************************************
		' * Summary:
		' * During the manufacturing process, various pieces of
		' * configuration and calibration data are written to the OTP
		' * memory onboard the DS2406 that resides on this analog output
		' * device. This function demonstrates how to parse this
		' * information.
		' * 
		' * Description:
		' * The following table describes the various pieces of data
		' * stored in the DS2406 that controls this analog output device.
		' * This data was written into the DS2406 during the product's
		' * manufacturing process.
		' * <TABLE>
		' * Address   Page   Size       Function       \Example     \Description
		' *                                             (hex)        
		' * --------  -----  ---------  -------------  -----------  ---------------------------------------
		' * 32        1      1          UDP length     1D           UDP packet length
		' * 33        1      4          Device ID      414F5554     Used to determine type of device.<P>
		' *                                                          "AOUT" - AnalogOutput.
		' * 61        1      1          UDP            0            UDP continuation pointer
		' *                              continuation                
		' *                              pointer                     
		' * 62        1      2          UDP CRC 16     9D9E         UDP CRC16 error check
		' * 64        2      1          UDP length     1D           UDP packet length
		' * 72        2      2          Poll time      3A98         Recommended poll time<P>
		' *                                                          Ex.: 15000 ms
		' * 74        2      Variable   Label Name     5A325445CE   Label name of the device. (variable
		' *                                                          length with 8th bit termination)<P>
		' *                                                          Ex.: "Z2TEN"
		' * ??        2      Variable   Units Label    566F6C74F3   Units Label (variable length with 8th
		' *                                                          bit termination)<P>
		' *                                                          Ex.: "Volts"
		' * ??        2(3)   Variable   Reserved       6E6F6EE5     Label reserved for future use
		' *                              Label                       (variable length with 8th bit
		' *                                                          termination)<P>
		' *                                                          Ex.: "none"
		' * ??        2(3)   Variable   Engineering    31302EB0     Full scale reading in engineering
		' *                              Scale                       units at 32767. (See note below for
		' *                                                          format)<P>
		' *                                                          Ex.: "10.0"
		' * ??        2(3)   Variable   Engineering    302EB0       Engineering Units offset. (see note
		' *                              Offset                      below for format)<P>
		' *                                                          Ex.: "0.0"
		' * ??        2(3)   1          UDP            00           UDP continuation pointer (typically 0)
		' *                              continuation                
		' *                              pointer                     
		' * ??        2(3)   2          UDP CRC-16     064F         UDP CRC16 error check
		' * \                                                       
		' * </TABLE>
		' * Notes:
		' * 
		' *       1. All strings have the last character's 8th bit set to
		' *          1 to indicate end of string.
		' *       2. If a string field is "empty", a 0x80 is placed for
		' *          termination.
		' *       3. Engineering Scale and Engineering Offset are real
		' *          numbers represented as strings with the 8th bit set on the
		' *          last character for termination.
		' * 
		' * See Also:
		' * initializeAnalogOutput()
		' * 
		' * Parameters:
		' * rawConfigData :  char() of data representing the memory pages
		' *                  previously read from the DS2406.                                              
		' *************************************************************************************************
		Private Sub parseConfigurationData(ByVal rawConfigData As Char())

			Dim byteCount As Integer
			Dim tempData As String
			Dim origData As String
			Dim startByteCount As Integer

			Me.log("Now Parse the configuration data read from the DS2406.")
			'Parse the PollTime
			byteCount = 0
			tempData = ""
			tempData = Hex2(AscW(rawConfigData(72))) & Hex2(AscW(rawConfigData(73)))
			Me.pollTime = CDec("&h" & tempData)
			Me.log("Poll Time: " & Me.pollTime & " mS" & " ... Parsed from bytes 72-73... (" & tempData & ")")

			'Parse the LabelName
			byteCount = 0
			startByteCount = byteCount
			tempData = ""
			origData = ""
			Do While (AscW(rawConfigData(74 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(74 + byteCount)
				origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)))
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(74 + byteCount)) And &H7F))
			origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)) And &H7F)
			Me.labelName = tempData
			Me.log("Label Name: " & Me.labelName & " ... Parsed from bytes " & CStr(74 + startByteCount) & "-" & CStr(74 + byteCount) & " (" & origData & ")")

			'Parse the UnitsLabel
			tempData = ""
			origData = ""
			byteCount = byteCount + 1
			startByteCount = byteCount
			Do While (AscW(rawConfigData(74 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(74 + byteCount)
				origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)))
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(74 + byteCount)) And &H7F))
			origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)) And &H7F)
			Me.unitsLabel = tempData
			Me.log("Units Label: " & Me.unitsLabel & " ... Parsed from bytes " & CStr(74 + startByteCount) & "-" & CStr(74 + byteCount) & " (" & origData & ")")

			'Parse the ReservedLabel
			tempData = ""
			origData = ""
			byteCount = byteCount + 1
			startByteCount = byteCount
			Do While (AscW(rawConfigData(74 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(74 + byteCount)
				origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)))
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(74 + byteCount)) And &H7F))
			origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)) And &H7F)
			Me.reservedLabel = tempData
			Me.log("Reserved Label: " & Me.reservedLabel & " ... Parsed from bytes " & CStr(74 + startByteCount) & "-" & CStr(74 + byteCount) & " (" & origData & ")")

			'Parse the Engineering Scale
			tempData = ""
			origData = ""
			byteCount = byteCount + 1
			startByteCount = byteCount
			Do While (AscW(rawConfigData(74 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(74 + byteCount)
				origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)))
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(74 + byteCount)) And &H7F))
			origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)) And &H7F)
			Me.engrScale = CDec(tempData)
			Me.log("Engineering Scale: " & Me.engrScale & " ... Parsed from bytes " & CStr(74 + startByteCount) & "-" & CStr(74 + byteCount) & " (" & origData & ")")

			'Parse the Engineering Offset
			tempData = ""
			origData = ""
			byteCount = byteCount + 1
			startByteCount = byteCount
			Do While (AscW(rawConfigData(74 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(74 + byteCount)
				origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)))
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(74 + byteCount)) And &H7F))
			origData = origData & Hex2(AscW(rawConfigData(74 + byteCount)) And &H7F)
			Me.engrOffset = CDec(tempData)
			Me.log("Engineering Offset: " & Me.engrOffset & " ... Parsed from bytes " & CStr(74 + startByteCount) & "-" & CStr(74 + byteCount) & " (" & origData & ")")
		End Sub

		' **************************************************************
		' * Summary:
		' * During the manufacturing process, the EDS analog output
		' * devices are "tagged" by writing the bytes "AOUT" into the
		' * first memory page of the onboard DS2406. By examining this
		' * tagging information, you can easily distinguish this device
		' * from common DS2406's. This function demonstrates how to look
		' * for this tagging information.                               
		' **************************************************************
		Public Shared Function getEDSAnalogOutputDeviceType(ByVal testDS2406 As DS2406) As String
			Dim retVal As String
			Dim ds2406MemoryPages As ArrayList
			Dim memoryPage As Char()
			ds2406MemoryPages = testDS2406.ReadMemoryPages(1, 1)
			Dim hexPage As String = ds2406MemoryPages(0)

			memoryPage = EDS_OWInterface.Hex2CharArray(hexPage)
			If New String(memoryPage, 1, 4).CompareTo("AOUT") = 0 Then
				retVal = New String(memoryPage, 1, 4)			 '4 Character Device ID
			Else
				retVal = ""
			End If

			Return retVal
		End Function

		Public Function createDocumentation() As String
			Dim docs As String
			'Read configuration / Calibration data
			Me.docs = ""
			Me.creatingDocumentation = True

			Me.log("The " & Me.labelName & " is a member of the EDS Analog Ouput family of devices.")
			Me.log("This analog output class represents a family of devices that contain an EDS 12-bit analog output module that has been outfitted with a Dallas Semiconductor 1-wire front-end. By implementing this configuration, Embedded Data Systems is able to put a 1-Wire front end on a wide variety of output devices that would not otherwise be accessible to the 1-Wire bus. In this fashion, you only have to implement the interface once in your software in order to communicate to a wide variety of analog output devices.")
			Me.log("The EDS analog outputs incorporate a DS2406 Addressable Switch which sits on the 1-Wire bus and provides a 1-Wire gateway to the 12-bit analog subsystem. The DS2406 is also used to provide each device with unique identification, and storage of calibration information and other usage information.")
			Me.log("Unique device identification and calibration information is stored in the OTP memory of the DS2406. This information can include, among other things:")
			Me.log("	Device ID - Used to determine the type of device")
			Me.log("	PollTime - Desired Polltime")
			Me.log("	LabelName - Label name of the device Ex.: 'Z2TEN'")
			Me.log("	Units Label - Units label Ex.: 'Volts'")
			Me.log("	Calibration Table - Provides enough data to allow the application to convert from the raw binary units returned by the analog subsystem to engineering units.")
			Me.log("All of this data is written to the DS2406's OTP memory during the manufacturing process of the device.")
			Me.log("")
			Me.log("General usage is as follows:")
			Me.log("<b>To set the analog output:</b>")
			Me.log("The DS2406 is addressed and then setup to manipulate the PIO_A pin. Data is then written to the digital-to-analog converter using PIO_A. The value sent to the analog output is a 16 bit signed binary value (2's compliment).")
			Me.log("<b>To read the analog output:</b>")
			Me.log("The DS2406 is addressed and then setup to manipulate the PIO_A channel in order to read information from the analog-to-digital converter. The value received from the analog output is a 16 bit signed binary value (2's compliment)")
			Me.log("The bus master can use the calibration information contained in the OTP to convert the 12-bit analog reading to a calibrated engineering unit.")
			Me.log("")
			Me.log("The following are annotated examples of each primary operation, performed on the " & Me.labelName & " with ROMId " & Me.PrimaryDS2406.ROMId & " when accessed via an EDS 1-Wire bus master.")

			Me.log("<B>Initializing Analog Output Device</b>")
			initializeAnalogOutput()


			Me.log("")
			Me.log("<B>Setting Current Ouput Level</b>")
			Me.setValue(9.99)

			Me.log("")
			Me.log("<B>Reading Current Ouput Level</b>")
			readValue()

			'Read Existing value
			'Set New Value
			'Read New Value

			Return Me.docs
			Me.creatingDocumentation = False
		End Function
		Private Function log(ByVal whatToLog As String)
			If creatingDocumentation Then
				Me.docs = Me.docs & whatToLog & "<br>" & vbCrLf
			End If
		End Function

	End Class