
' The Embedded Data Systems' analog probe represents a family
' \of devices that contain an EDS 12-bit analog input module
' that has been outfitted with a Dallas Semiconductor 1-wire
' front-end. The back-end of these devices are then connected
' to a wide variety of sensors. By implementing this
' configuration, Embedded Data Systems is able to put a 1-Wire
' front end on a wide variety of sensors that would not
' \otherwise be accessible to the 1-Wire bus. Just as
' interesting, you only have to implement the interface once in
' your software in order to communicate to a wide variety of
' sensors.
' 
' Embedded Data Systems' sensors that currently support this
' interface are as follows:
' 
'     * HMP2001 - Relative Humidity / Temperature Sensor
'     * BAR2001 - Barometric Pressure / Temperature Sensor
'     * PH Probes
'     * Weight Scale Probes
'     * Conductivity Probes
' 
' 
' 
' The EDS analog probes incorporate a DS2406 Addressable Switch
' that sits on the 1-Wire bus and provides a 1-Wire gateway to
' the 12-bit analog subsystem. The DS2406 is also used to
' provide each device with unique identification, and storage
' \of calibration information and other usage information.
' 
' Unique device identification and calibration information is
' stored in the OTP memory of the DS2406. This information can
' include, among other things:
' 
'     * Device ID - Used to determine the type of device
'     * DS18S20 Address - Many of these devices have an onboard
'       DS18S20 to allow for temperature compensated readings. This
'       address allows you to associate the DS18S20 with this
'       particular device.
'     * Number of Samples - Recommended number of samples to
'       read from the analog input during each conversion.
'     * Flags - Used to indicate whether the probe is
'       parasitically powered, etc.
'     * Pre-Dwell Time - Time in milliseconds that the
'       application should assert Stong Pullup to give extra power to
'       the probe during conversions.
'     * Calibration Table - Provides enough data to allow the
'       application to convert from the raw binary units returned by
'       the analog subsystem to engineering units. Also provides
'       temperature coefficients and calibration data if the readings
'       are to be temperature compenstated.
' 
' All of this data is written to the DS2406's OTP memory during
' the manufacturing process of the device.
' 
' 
' 
' General usage is a follows:
' 
' The DS2406 is addressed and then setup so that the I/O lines
' can be controlled. Following the protocol, a bus master can
' then cause the DS2406 to trigger the analog subsystem into
' starting a conversion process and then obtain the analog data
' through manipulations of the DS2406 I/O lines.
' 
' 
' 
' When probes are not externally powered (i.e. parasitically
' powered from the MicroLan network), the bus master must
' assert a Strong Pullup and not communicate on the bus for
' specified times to power the probe. Some of these times are
' stored in the DS2406's OTP; other times are specific for each
' probe type.
' 
' 
' 
' The bus master can use the calibration information contained
' in the OTP to convert the 12-bit analog reading to a
' calibrated engineering unit.                                       
	Public Class EDSAnalogProbe

		Public PrimaryDS2406 As DS2406
		Private AssociatedDS18S20 As DS18S20
		Private calib1Engr As Decimal
		Private calib1Raw As Integer
		Private calib2Engr As Decimal
		Private calib2Raw As Integer
		Private tempCalib As Decimal
		Private tempCoeff As Integer
		Private lastTemperature As Double
		Private lastEngineeringValue As Double

		Public Sub New(ByVal primaryDS2406 As DS2406)
			Me.PrimaryDS2406 = primaryDS2406
			initializeAnalogProbe()
		End Sub
		'Initiates a temperature conversion on the on-board DS18S20
		Public Sub ReadTemperature()
			Me.AssociatedDS18S20.ConvertTemperature()
		End Sub
		'Returns the temperature read during the last temperature conversion
		Public Function GetLastTemperature(Optional ByVal Farenheight As Boolean = False) As Decimal
			Return Me.AssociatedDS18S20.GetLastTemperature(Farenheight)
		End Function
		'Initiates a temperature conversion and returns the sampled temperature.
		'Effectively combines ReadTemperature() and GetLastTemperature() into a single call.
		Public Function GetTemperature(Optional ByVal Farenheight As Boolean = False) As Decimal
			Me.lastTemperature = Me.AssociatedDS18S20.GetTemperature(Farenheight)
			Return Me.lastTemperature
		End Function
		' Finds associated temp sensor, and initializes calibration data
		Private Sub initializeAnalogProbe()
			Const pageLength As Byte = 32
			Dim memoryPages As ArrayList
			Dim memoryPage As Char()
			Dim memoryBuffer(128) As Char

			'Read the configuration data out of the DS2406's memory pages
			'and store into a single continuous char()
			memoryPages = Me.PrimaryDS2406.ReadMemoryPages(0, 4)
			For t As Integer = 0 To 3
				memoryPage = EDS_OWInterface.Hex2CharArray(memoryPages(t))
				For position As Integer = 0 To 31
					memoryBuffer((t * 32) + position) = memoryPage(position)
				Next
			Next
			'Now parse the probe's configuration data out the the char()
			parseConfigurationData(memoryBuffer)
		End Sub

		' Reads the current analog value from the sensor, performs 
		' temperature compensation on the reading.
		Public Function readValue() As Decimal
			Dim owresults As String
			Dim byteResults As Char()
			Dim analogValue As Integer
			Dim analogValueTempCompensated As Integer
			Dim engineeringValue As Decimal
			Dim scale As Decimal
			Dim offset As Decimal
			Dim readAnalog As Byte() = New Byte(3) {&HFE, &HFF, &HFF, &HFF}

			Me.GetTemperature()
			Me.PrimaryDS2406.BusMaster.OW_GetLock()
			Me.PrimaryDS2406.BusMaster.OW_AddressDevice(Me.PrimaryDS2406.ROMId)
			owresults = Me.PrimaryDS2406.ChannelAccess(readAnalog, True, False, DS2406.CRC_EVERY_8_BYTES, DS2406.CHANNEL_A_ONLY, True, False)

			System.Threading.Thread.Sleep(100)

			Me.PrimaryDS2406.BusMaster.OW_ResetBus()
			Me.PrimaryDS2406.BusMaster.OW_AddressDevice(Me.PrimaryDS2406.ROMId)
			owresults = Me.PrimaryDS2406.ChannelAccess(readAnalog, True, False, DS2406.CRC_EVERY_8_BYTES, DS2406.CHANNEL_A_ONLY, True, False)
			Me.PrimaryDS2406.BusMaster.OW_ReleaseLock()

			byteResults = EDS_OWInterface.Hex2CharArray(owresults)
			analogValue = parseAnalogData(AscW(byteResults(9)), AscW(byteResults(11)))
			analogValueTempCompensated = ((analogValue * (1 + (Me.lastTemperature - Me.tempCalib) * (Me.tempCoeff / 1000000))))
			scale = (calib2Engr - calib1Engr) / (calib2Raw - calib1Raw)
			offset = calib1Engr - (calib1Raw * scale)
			engineeringValue = (analogValueTempCompensated * scale) + offset
			Me.lastEngineeringValue = engineeringValue

			Return engineeringValue

		End Function

		' Parses the configuration / calibration data from the DS2406 
		' The following table details where these parameters exist in the DS2406 memory. */

		'/** DS2406 ROM Data For Analog Probes
		' * Note that sometimes the calibration information overlaps into page 3.
		' * Addr	Size	Function	        ExIn hex	        Description
		' * 33   4     Device ID	        52485248	        Used to determine type of device.  'RHRH' - relative humidity // ** Device ID field **
		' * 64   8     DS18S20 ROM Addr  810000001D500D10	ROM address of the DS18S20 attached. // ** Temperature sensor ROM code **
		' * Calibration Table
		' * 79   --    Calib1Engr^       B0                First calibration in engineering units Example: 0
		' *    	2     Calib1Raw         02CD              First calibration in raw binary units Example: 717  
		' *      ---   Calib2Engr^       3130B0            Second calibration in engineering units Example: 100
		' *      2     Calib2Raw         0C39              Second calibration in raw binary units Example: 3129
		' *      ---   TempCalib^        32352EB0          Calibration Temperature for compensation Example: 25.0
		' *      2     TempCoeff         0866              Temperature Coefficient for compensation Example: 2150
		' *      1     Table Terminator	FF                Terminates the calibration table
		' *      * ^ Designates a Real Number field.  Real Number fields are ASCII strings
		' *      * with no null terminator but with the last characters most significant
		' *      * bit set to indicate end of string.
		' */
		Private Sub parseConfigurationData(ByVal rawConfigData As Char())

			' Parse the Associated Temperature Sensor's Serial Number
			Dim rawSerialData(16) As Char
			Dim byteCount As Integer
			Dim tempData As String

			For byteCount = 0 To 7
				rawSerialData(byteCount) = rawConfigData(64 + byteCount)
			Next
			Dim DS18S20SerialNumber As New String(parseSerialNumber(rawSerialData))
			Me.AssociatedDS18S20 = New DS18S20(DS18S20SerialNumber, PrimaryDS2406.BusMaster)

			'Parse the 1st Calibration point
			byteCount = 0
			tempData = ""
			Do While (AscW(rawConfigData(79 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(79 + byteCount)
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(79 + byteCount)) And &H7F))
			Me.calib1Engr = CDec(tempData)
			byteCount = byteCount + 1

			tempData = Hex2(AscW(rawConfigData(79 + byteCount)))
			byteCount = byteCount + 1
			tempData = tempData & Hex2(AscW(rawConfigData(79 + byteCount)))
			byteCount = byteCount + 1
			Me.calib1Raw = CInt("&h" & tempData)

			'Parse the 2nd Calibration point
			tempData = ""
			Do While (AscW(rawConfigData(79 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(79 + byteCount)
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(79 + byteCount)) And &H7F))
			Me.calib2Engr = CDec(tempData)
			byteCount = byteCount + 1

			tempData = Hex2(AscW(rawConfigData(79 + byteCount)))
			byteCount = byteCount + 1
			tempData = tempData & Hex2(AscW(rawConfigData(79 + byteCount)))
			byteCount = byteCount + 1
			Me.calib2Raw = CInt("&h" & tempData)

			'Parse the Temperature/Coefficient used for Temperature Compensation
			tempData = ""
			Do While (AscW(rawConfigData(79 + byteCount)) And 128) = 0
				tempData = tempData & rawConfigData(79 + byteCount)
				byteCount = byteCount + 1
			Loop
			tempData = tempData & Chr((AscW(rawConfigData(79 + byteCount)) And &H7F))
			Me.tempCalib = CDec(tempData)
			byteCount = byteCount + 1

			tempData = Hex2(AscW(rawConfigData(79 + byteCount)))
			byteCount = byteCount + 1
			tempData = tempData & Hex2(AscW(rawConfigData(79 + byteCount)))
			byteCount = byteCount + 1
			Me.tempCoeff = CInt("&h" & tempData)
		End Sub

		'Parses a Dallas 16 Byte ROMId from the raw data stored in the 
		'DS2406
		Private Function parseSerialNumber(ByVal rawSerialData As Char()) As String
			Dim nextPair As String
			Dim parsedSerialNumber As String

			For byteCount As Integer = 0 To 7
				'nextPair = hexConverter.toHexString( ((int) rawSerialData[byteCount]) & 0xFF );
				nextPair = Hex2(AscW(rawSerialData(byteCount)))
				parsedSerialNumber = parsedSerialNumber & nextPair
			Next
			Return parsedSerialNumber
		End Function
		' Returns a numerical value that the OW bytes represent
		Private Function parseAnalogData(ByVal owHighByte As Byte, ByVal owLowByte As Byte) As Integer
			Dim retval As Integer = 0

			'Ones Compliment
			Dim low As Byte = (Not owLowByte)
			Dim high As Byte = (Not owHighByte)
			'Reverse the high byte bits
			For i As Integer = 0 To 7
				retval <<= 1
				retval = retval Or high And &H1
				high >>= 1
			Next
			'Reverse the low 4 bits
			For i As Integer = 0 To 3
				retval <<= 1
				retval = retval Or low And &H1
				low >>= 1
			Next
			Return retval
		End Function

		'Analyses the memory of the given DS2406 to determine if it represents
		'an EDS analog probe.  If so, returns a string representing the device type.
		Public Shared Function getEDSAnalogProbeDeviceType(ByVal testDS2406 As DS2406) As String
			Dim retVal As String
			Dim tempString As String		 'TODO - Remove this
			Dim ds2406MemoryPages As ArrayList
			Dim memoryPage As Char()
			ds2406MemoryPages = testDS2406.ReadMemoryPages(1, 1)
			Dim hexPage As String = ds2406MemoryPages(0)
			memoryPage = EDS_OWInterface.Hex2CharArray(hexPage)
			tempString = New String(memoryPage, 10, 4)		 'TODO - Remove this
			If New String(memoryPage, 10, 4).CompareTo("#M5Z") = 0 Then
				retVal = New String(memoryPage, 1, 4)			'4 Character Device ID
			Else
				retVal = ""
			End If
			Return retVal
		End Function

	End Class