
' *******************************************************************************************************
' * Description:
' * The EDS CT1 Counter is designed to count and maintain switch closure pulses from dry contact devices
' * such as relays and magnetic reed switches. While a DS2423 counter chip is onboard, the CT1 is not to
' * be confused with a simple DS2423. In addition to the counter function, the CT1 actually maintains a
' * 32-bit time stamp with the last three transitions to the active (closed) state. The most recent three
' * counts and times are stored, most recent first, in memory. The user can determine rate, total time
' * and total count at any time with a single read of the TMEX file format compatible RAM page 2. The CT1
' * Counter provides the necessary switch excitation voltage so that no external power source is
' * required. CAT-5 cable, provides electrical connection to the 1-WireT network. All I/O and data
' * communications with the sensor can be CRC16 error checked for accuracy. The data is stored in RAM
' * memory as a TMEX data packed. The CT1 Counter has a built-in multidrop controller (DS2423), which
' * provides a unique 64 bit registration number (8-bit family code + 48 bit serial number + 8 bit CRC)
' * assuring error-free selection and absolute identity; no two parts are alike. Unique addressing allows
' * I/O function to be identified absolutely, no more dip switch address confusion. All unique counter
' * identification and calibration data is stored in on-board ROM memory for easy installation and setup.
' * 
' * 
' * Note:
' * Ironically, while the CT1 Counter contains a DS2423 1-Wire counter chip onboard, it is not used in
' * the counting function. It is simply used to provide a unique ID, and to provide storage space for
' * both the tagging information that allows you to identify the device as a CT1, and for the CT1's
' * \onboard micros to write their count and timestamp data into. This data is written to the DS2423's
' * SRAM when you are not accessing it. If you attempt to read counter data from the DS2423 in the
' * traditional way, you will not read valid pulse count data for the CT1.
' * 
' * The CT1 Counter data record contains the three most recent time/count values. This consists of 3
' * 8-byte fields starting with the most recent time/count value each in 32-bit format. The data is
' * stored in the RAM memory of a DS2423 in page 2, memory location 0040 Hex. The record is organized as
' * a TMEX file record with byte count, data, continuation pointer and CRC-16 for a total of 28 bytes.
' * This format can be easily read with the HA5 file memory read command. The example below illustrates
' * the how this is done and how to interpret the data response.
' * <code>
' * HA5 Command Response
' * "aF1D\<cr\>" "970000000124DB1D\<CR\>"
' * "aL,0102\<cr\>" "00001E4E0000000900001E0D0000000800001E0600000007"
' * </code>
' * The read file record command removes the byte count, continuation pointer and CRC-16 and performs a
' * CRC-16 error check automatically. The received record is explained below:
' * <code>
' * The HA5 received record: 00001E4E0000000900001E0D0000000800001E0600000007
' * </code>
' * Consists of three sub fields of the most recent time/count values in 32-bit format. The fields
' * consist of 8 bytes (16 ASCII Hex characters) each starting with the 4-byte time and ending with the
' * 4-byte count.
' * 
' * <code>
' * First Field: <b><u>00001E4E00000009</u></b>00001EOD0000000800001E0600000007
' * </code>
' * This first 4-byte sub-field (00001E4E) is the time in 5 millisecond units that has expired between
' * the most recent count and the power up of the CT-1. The on-board clock will rollover every 248.5
' * days. The second sub-field (00000009) is the most recent count. These values are in hexadecimal
' * format MSB first.
' * 
' * This value represents 38.79 seconds and a count of 9.
' * 
' * <code>
' * Second: 00001E4E00000009<b><u>00001E0D00000008</u></b>00001E0600000007
' * </code>
' * This second field is the time and count of the count value that occurred before the most recent;
' * first count. The time is 38.46 seconds and the count is 8. The user can use this difference time and
' * difference count to determine the rate of the input pulses at the most recent count.
' * 
' * 
' * 
' * The third sub-field is the count and time of the input that occurred before the second event. As new
' * events occur the subfields are moved from left to right such that the most recent is always first.
' * The user may take data at any time but there must be at least 100 milliseconds between reads of the
' * RAM record.                                                                                          
' *******************************************************************************************************
	Public Class EDS_CT1Counter

		Public primaryDS2423 As DS2423

		'Analyses the memory of the given DS2423 to determine if it represents
		'an EDS CT1 Counter device.  If so, returns a string representing the device type.
		Public Shared Function getEDSCounterType(ByVal testDS2423 As DS2423) As String
			Dim retVal As String
			Dim response As OW_Response

			response = testDS2423.BusMaster.OW_ReadFileRecords(2, 1, testDS2423.ROMId)
			If CInt(response.Exception_Code) = 0 Then
				'The data residing a memory page 2 represents a valid TMEX formatted record,
				'so this device is likely a CT1 counter.
				retVal = "CT1"
			Else
				retVal = ""
			End If
			Return retVal
		End Function

		' **************************************************************
		' * Summary:
		' * Default constructor for EDS_CT1Counter
		' * 
		' * Description:
		' * Requires passing in a reference to the primary DS2423 that
		' * provides the data storage on the device. This
		' * DS2423 might have been previously discovered using
		' * EDS_CT1Counter.getEDScounterType().
		' 
		' **************************************************************
		Public Sub New(ByVal primaryDS2423 As DS2423)
			Me.PrimaryDS2423 = primaryDS2423
		End Sub


		Public Function getCurrentCount(ByVal lockid As String) As Long
			Dim localLock As Integer
			Dim response As OW_Response
			Dim retVal As Long
			If IsNothing(lockid) Then
				'The bus isn't already locked, so acquire a lock.
				lockid = Me.primaryDS2423.BusMaster.OW_GetLock().Data(0)
				localLock = True
			Else
				'The bus is already locked and being managed by the caller.
				localLock = False
			End If

			response = Me.primaryDS2423.BusMaster.OW_ReadFileRecords(2, 1, Me.primaryDS2423.ROMId)
			Me.primaryDS2423.BusMaster.OW_ResetBus()


			If localLock Then
				Me.primaryDS2423.BusMaster.OW_ReleaseLock()
			End If

			If CInt(response.Exception_Code) = 0 Then
				'The data residing at memory page 2 represents a valid TMEX formatted record,
				retVal = CLng("&H" & Mid$(response.Data(0), 9, 8))
			End If
			Return retVal

		End Function


	End Class